@@ -1,6 +1,7 @@
 use std::{env, path::Path, str::FromStr};
 
 use async_trait::async_trait;
+use atuin_common::utils;
 use chrono::{prelude::*, Utc};
 use fs_err as fs;
 use itertools::Itertools;
@@ -13,16 +14,27 @@ use sqlx::{
 };
 
 use super::{
-    event::{Event, EventType},
     history::History,
     ordering,
     settings::{FilterMode, SearchMode},
 };
 
 pub struct Context {
-    session: String,
-    cwd: String,
-    hostname: String,
+    pub session: String,
+    pub cwd: String,
+    pub hostname: String,
+}
+
+#[derive(Default, Clone)]
+pub struct OptFilters {
+    pub exit: Option<i64>,
+    pub exclude_exit: Option<i64>,
+    pub cwd: Option<String>,
+    pub exclude_cwd: Option<String>,
+    pub before: Option<String>,
+    pub after: Option<String>,
+    pub limit: Option<i64>,
+    pub offset: Option<i64>,
 }
 
 pub fn current_context() -> Context {
@@ -31,10 +43,7 @@ pub fn current_context() -> Context {
         std::process::exit(1);
     };
     let hostname = format!("{}:{}", whoami::hostname(), whoami::username());
-    let cwd = match env::current_dir() {
-        Ok(dir) => dir.display().to_string(),
-        Err(_) => String::from(""),
-    };
+    let cwd = utils::get_current_dir();
 
     Context {
         session,
@@ -44,7 +53,7 @@ pub fn current_context() -> Context {
 }
 
 #[async_trait]
-pub trait Database: Send + Sync {
+pub trait Database: Send + Sync + 'static {
     async fn save(&mut self, h: &History) -> Result<()>;
     async fn save_bulk(&mut self, h: &[History]) -> Result<()>;
 
@@ -64,23 +73,30 @@ pub trait Database: Send + Sync {
 
     async fn update(&self, h: &History) -> Result<()>;
     async fn history_count(&self) -> Result<i64>;
-    async fn event_count(&self) -> Result<i64>;
-    async fn merge_events(&self) -> Result<i64>;
 
     async fn first(&self) -> Result<History>;
     async fn last(&self) -> Result<History>;
     async fn before(&self, timestamp: chrono::DateTime<Utc>, count: i64) -> Result<Vec<History>>;
 
+    async fn delete(&self, mut h: History) -> Result<()>;
+    async fn deleted(&self) -> Result<Vec<History>>;
+
+    // Yes I know, it's a lot.
+    // Could maybe break it down to a searchparams struct or smth but that feels a little... pointless.
+    // Been debating maybe a DSL for search? eg "before:time limit:1 the query"
+    #[allow(clippy::too_many_arguments)]
     async fn search(
         &self,
-        limit: Option<i64>,
         search_mode: SearchMode,
         filter: FilterMode,
         context: &Context,
         query: &str,
+        filter_options: OptFilters,
     ) -> Result<Vec<History>>;
 
     async fn query_history(&self, query: &str) -> Result<Vec<History>>;
+
+    async fn all_with_count(&self) -> Result<Vec<(History, i32)>>;
 }
 
 // Intended for use on a developer machine and not a sync server.
@@ -120,31 +136,10 @@ impl Sqlite {
         Ok(())
     }
 
-    async fn save_event(tx: &mut sqlx::Transaction<'_, sqlx::Sqlite>, e: &Event) -> Result<()> {
-        let event_type = match e.event_type {
-            EventType::Create => "create",
-            EventType::Delete => "delete",
-        };
-
-        sqlx::query(
-            "insert or ignore into events(id, timestamp, hostname, event_type, history_id)
-                values(?1, ?2, ?3, ?4, ?5)",
-        )
-        .bind(e.id.as_str())
-        .bind(e.timestamp.timestamp_nanos())
-        .bind(e.hostname.as_str())
-        .bind(event_type)
-        .bind(e.history_id.as_str())
-        .execute(tx)
-        .await?;
-
-        Ok(())
-    }
-
     async fn save_raw(tx: &mut sqlx::Transaction<'_, sqlx::Sqlite>, h: &History) -> Result<()> {
         sqlx::query(
-            "insert or ignore into history(id, timestamp, duration, exit, command, cwd, session, hostname)
-                values(?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8)",
+            "insert or ignore into history(id, timestamp, duration, exit, command, cwd, session, hostname, deleted_at)
+                values(?1, ?2, ?3, ?4, ?5, ?6, ?7, ?8, ?9)",
         )
         .bind(h.id.as_str())
         .bind(h.timestamp.timestamp_nanos())
@@ -154,6 +149,7 @@ impl Sqlite {
         .bind(h.cwd.as_str())
         .bind(h.session.as_str())
         .bind(h.hostname.as_str())
+        .bind(h.deleted_at.map(|t|t.timestamp_nanos()))
         .execute(tx)
         .await?;
 
@@ -161,6 +157,8 @@ impl Sqlite {
     }
 
     fn query_history(row: SqliteRow) -> History {
+        let deleted_at: Option<i64> = row.get("deleted_at");
+
         History {
             id: row.get("id"),
             timestamp: Utc.timestamp_nanos(row.get("timestamp")),
@@ -170,6 +168,7 @@ impl Sqlite {
             cwd: row.get("cwd"),
             session: row.get("session"),
             hostname: row.get("hostname"),
+            deleted_at: deleted_at.map(|t| Utc.timestamp_nanos(t)),
         }
     }
 }
@@ -178,11 +177,8 @@ impl Sqlite {
 impl Database for Sqlite {
     async fn save(&mut self, h: &History) -> Result<()> {
         debug!("saving history to sqlite");
-        let event = Event::new_create(h);
-
         let mut tx = self.pool.begin().await?;
         Self::save_raw(&mut tx, h).await?;
-        Self::save_event(&mut tx, &event).await?;
         tx.commit().await?;
 
         Ok(())
@@ -194,9 +190,7 @@ impl Database for Sqlite {
         let mut tx = self.pool.begin().await?;
 
         for i in h {
-            let event = Event::new_create(i);
             Self::save_raw(&mut tx, i).await?;
-            Self::save_event(&mut tx, &event).await?;
         }
 
         tx.commit().await?;
@@ -221,7 +215,7 @@ impl Database for Sqlite {
 
         sqlx::query(
             "update history
-                set timestamp = ?2, duration = ?3, exit = ?4, command = ?5, cwd = ?6, session = ?7, hostname = ?8
+                set timestamp = ?2, duration = ?3, exit = ?4, command = ?5, cwd = ?6, session = ?7, hostname = ?8, deleted_at = ?9
                 where id = ?1",
         )
         .bind(h.id.as_str())
@@ -232,6 +226,7 @@ impl Database for Sqlite {
         .bind(h.cwd.as_str())
         .bind(h.session.as_str())
         .bind(h.hostname.as_str())
+        .bind(h.deleted_at.map(|t|t.timestamp_nanos()))
         .execute(&self.pool)
         .await?;
 
@@ -332,49 +327,15 @@ impl Database for Sqlite {
         Ok(res)
     }
 
-    async fn event_count(&self) -> Result<i64> {
-        let res: i64 = sqlx::query_scalar("select count(1) from events")
-            .fetch_one(&self.pool)
+    async fn deleted(&self) -> Result<Vec<History>> {
+        let res = sqlx::query("select * from history where deleted_at is not null")
+            .map(Self::query_history)
+            .fetch_all(&self.pool)
             .await?;
 
         Ok(res)
     }
 
-    // Ensure that we have correctly merged the event log
-    async fn merge_events(&self) -> Result<i64> {
-        // Ensure that we do not have more history locally than we do events.
-        // We can think of history as the merged log of events. There should never be more history than
-        // events, and the only time this could happen is if someone is upgrading from an old Atuin version
-        // from before we stored events.
-        let history_count = self.history_count().await?;
-        let event_count = self.event_count().await?;
-
-        if history_count > event_count {
-            // pass an empty context, because with global listing we don't care
-            let no_context = Context {
-                cwd: String::from(""),
-                session: String::from(""),
-                hostname: String::from(""),
-            };
-
-            // We're just gonna load everything into memory here. That sucks, I know, sorry.
-            // But also even if you have a LOT of history that should be fine, and we're only going to be doing this once EVER.
-            let all_the_history = self
-                .list(FilterMode::Global, &no_context, None, false)
-                .await?;
-
-            let mut tx = self.pool.begin().await?;
-            for i in all_the_history.iter() {
-                // A CREATE for every single history item is to be expected.
-                let event = Event::new_create(i);
-                Self::save_event(&mut tx, &event).await?;
-            }
-            tx.commit().await?;
-        }
-
-        Ok(0)
-    }
-
     async fn history_count(&self) -> Result<i64> {
         let res: (i64,) = sqlx::query_as("select count(1) from history")
             .fetch_one(&self.pool)
@@ -385,11 +346,11 @@ impl Database for Sqlite {
 
     async fn search(
         &self,
-        limit: Option<i64>,
         search_mode: SearchMode,
         filter: FilterMode,
         context: &Context,
         query: &str,
+        filter_options: OptFilters,
     ) -> Result<Vec<History>> {
         let mut sql = SqlBuilder::select_from("history");
 
@@ -397,10 +358,14 @@ impl Database for Sqlite {
             .having("max(timestamp)")
             .order_desc("timestamp");
 
-        if let Some(limit) = limit {
+        if let Some(limit) = filter_options.limit {
             sql.limit(limit);
         }
 
+        if let Some(offset) = filter_options.offset {
+            sql.offset(offset);
+        }
+
         match filter {
             FilterMode::Global => &mut sql,
             FilterMode::Host => sql.and_where_eq("hostname", quote(&context.hostname)),
@@ -414,7 +379,7 @@ impl Database for Sqlite {
         match search_mode {
             SearchMode::Prefix => sql.and_where_like_left("command", query),
             SearchMode::FullText => sql.and_where_like_any("command", query),
-            SearchMode::Fuzzy => {
+            _ => {
                 // don't recompile the regex on successive calls!
                 lazy_static! {
                     static ref SPLIT_REGEX: Regex = Regex::new(r" +").unwrap();
@@ -460,6 +425,32 @@ impl Database for Sqlite {
             }
         };
 
+        filter_options
+            .exit
+            .map(|exit| sql.and_where_eq("exit", exit));
+
+        filter_options
+            .exclude_exit
+            .map(|exclude_exit| sql.and_where_ne("exit", exclude_exit));
+
+        filter_options
+            .cwd
+            .map(|cwd| sql.and_where_eq("cwd", quote(cwd)));
+
+        filter_options
+            .exclude_cwd
+            .map(|exclude_cwd| sql.and_where_ne("cwd", quote(exclude_cwd)));
+
+        filter_options.before.map(|before| {
+            interim::parse_date_string(before.as_str(), Utc::now(), interim::Dialect::Uk)
+                .map(|before| sql.and_where_lt("timestamp", quote(before.timestamp_nanos())))
+        });
+
+        filter_options.after.map(|after| {
+            interim::parse_date_string(after.as_str(), Utc::now(), interim::Dialect::Uk)
+                .map(|after| sql.and_where_gt("timestamp", quote(after.timestamp_nanos())))
+        });
+
         let query = sql.sql().expect("bug in search query. please report");
 
         let res = sqlx::query(&query)
@@ -478,6 +469,54 @@ impl Database for Sqlite {
 
         Ok(res)
     }
+
+    async fn all_with_count(&self) -> Result<Vec<(History, i32)>> {
+        debug!("listing history");
+
+        let mut query = SqlBuilder::select_from(SqlName::new("history").alias("h").baquoted());
+
+        query
+            .fields(&[
+                "id",
+                "max(timestamp) as timestamp",
+                "max(duration) as duration",
+                "exit",
+                "command",
+                "deleted_at",
+                "group_concat(cwd, ':') as cwd",
+                "group_concat(session) as session",
+                "group_concat(hostname, ',') as hostname",
+                "count(*) as count",
+            ])
+            .group_by("command")
+            .group_by("exit")
+            .and_where("deleted_at is null")
+            .order_desc("timestamp");
+
+        let query = query.sql().expect("bug in list query. please report");
+
+        let res = sqlx::query(&query)
+            .map(|row: SqliteRow| {
+                let count: i32 = row.get("count");
+                (Self::query_history(row), count)
+            })
+            .fetch_all(&self.pool)
+            .await?;
+
+        Ok(res)
+    }
+
+    // deleted_at doesn't mean the actual time that the user deleted it,
+    // but the time that the system marks it as deleted
+    async fn delete(&self, mut h: History) -> Result<()> {
+        let now = chrono::Utc::now();
+        h.command = String::from(""); // blank it
+        h.deleted_at = Some(now); // delete it
+
+        self.update(&h).await?; // save it
+
+        Ok(())
+    }
 }
 
 #[cfg(test)]
@@ -498,7 +537,17 @@ mod test {
             cwd: "/home/ellie".to_string(),
         };
 
-        let results = db.search(None, mode, filter_mode, &context, query).await?;
+        let results = db
+            .search(
+                mode,
+                filter_mode,
+                &context,
+                query,
+                OptFilters {
+                    ..Default::default()
+                },
+            )
+            .await?;
 
         assert_eq!(
             results.len(),
@@ -533,6 +582,7 @@ mod test {
             1,
             Some("beep boop".to_string()),
             Some("booop".to_string()),
+            None,
         );
         db.save(&history).await
     }
@@ -701,7 +751,15 @@ mod test {
         }
         let start = Instant::now();
         let _results = db
-            .search(None, SearchMode::Fuzzy, FilterMode::Global, &context, "")
+            .search(
+                SearchMode::Fuzzy,
+                FilterMode::Global,
+                &context,
+                "",
+                OptFilters {
+                    ..Default::default()
+                },
+            )
             .await
             .unwrap();
         let duration = start.elapsed();
