@@ -16,9 +16,26 @@ pub struct History {
     pub cwd: String,
     pub session: String,
     pub hostname: String,
+    pub deleted_at: Option<chrono::DateTime<Utc>>,
+}
+
+// Forgive me, for I have sinned
+// I need to replace rmp with something that is more backwards-compatible.
+// Protobuf, or maybe just json
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, sqlx::FromRow)]
+pub struct HistoryWithoutDelete {
+    pub id: String,
+    pub timestamp: chrono::DateTime<Utc>,
+    pub duration: i64,
+    pub exit: i64,
+    pub command: String,
+    pub cwd: String,
+    pub session: String,
+    pub hostname: String,
 }
 
 impl History {
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         timestamp: chrono::DateTime<Utc>,
         command: String,
@@ -27,6 +44,7 @@ impl History {
         duration: i64,
         session: Option<String>,
         hostname: Option<String>,
+        deleted_at: Option<chrono::DateTime<Utc>>,
     ) -> Self {
         let session = session
             .or_else(|| env::var("ATUIN_SESSION").ok())
@@ -43,6 +61,7 @@ impl History {
             duration,
             session,
             hostname,
+            deleted_at,
         }
     }
 
